==> n-rest-1.0.0.tgz: package/n-rest.js <==
var url_parse = require("url").parse;

function notFound(req, res, message) {

	message = (message || "Not Found\n") + "";

	res.writeHead(404, {
		"Content-Type": "text/plain",
		"Content-Length": Buffer.byteLength(message, "utf8")
	});

	if (req.method !== "HEAD") res.write(message);

	res.end();
}

exports.Server = function(logger) {

	logger = logger || console.log;

	var routes = [];

	function addRoute(method, pattern, handler, format) {

		if (typeof pattern === 'string') {

			pattern = new RegExp("^" + pattern + "$");
		}

		var route = {
			method: method,
			pattern: pattern,
			handler: handler
		};

		if (format !== undefined) {

			route.format = format;
		}

		routes.push(route);
	};

	function get(pattern, handler) {

		return addRoute("GET", pattern, handler);
	};
	function post(pattern, handler, format) {

		return addRoute("POST", pattern, handler, format);
	};
	function put(pattern, handler, format) {

		return addRoute("PUT", pattern, handler, format);
	};
	function del(pattern, handler) {

		return addRoute("DELETE", pattern, handler);
	};
	function head(pattern, handler) {

		return addRoute("HEAD", pattern, handler);
	};

	function resource(name, controller, format) {

		var parts = name.split("/");
		var size = parts.length;

		var ic_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];
			var _index = i;

			if(_index > 0 && size > 1) {

				ic_regexp += "/([^/]+)";
			}

			if(_index > 0 && size > 1) {

				ic_regexp += "/";
			}

			ic_regexp += _part;		
		}
		ic_regexp += ";?([A-Za-z_]*)";

		var edit_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];
			var _index = i;

			if(_index > 0 && size > 1) {

				edit_regexp += "/([^/]+)/";
			}

			edit_regexp += _part;
		};
		edit_regexp += "/([^/]+);?(edit)";

		var newing_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];
			var _index = i;

			if(_index > 0 && size > 1) {

				newing_regexp += "/([^/]+)/";
			}

			newing_regexp += _part;
		};
		newing_regexp += "/new$";

		var sud_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];
			var _index = i;

			if(_index > 0 && size > 1) {

				sud_regexp += "/([^/]+)/";
			}

			sud_regexp += _part;
		};
		sud_regexp += "/([^/]+)$";

		logger("get: "+ic_regexp +" --> index");
		logger("get: "+sud_regexp +" --> show");
		logger("get: "+edit_regexp +" --> edit");
		logger("get: "+newing_regexp +" --> newing");
		logger("post: "+ic_regexp +" --> create");
		logger("put: "+sud_regexp +" --> update");
		logger("delete: "+sud_regexp +" --> destroy");

	    get(new RegExp(edit_regexp), controller.edit);
	    get(new RegExp(newing_regexp), controller.newing);
	    get(new RegExp(sud_regexp), controller.show);
	    get(new RegExp(ic_regexp), controller.index);
		post(new RegExp(ic_regexp), controller.create, format);
		put(new RegExp(sud_regexp), controller.update, format);
		del(new RegExp(sud_regexp), controller.destroy);
	};

	function serve(req, res) {

		res.redirect = function redirect(location) {

			res.writeHead(302, {"Location": location});
			res.end();
		};

		res.innerRedirect = function innerRedirect(location) {

			logger("Internal Redirect: " + req.url + " -> " + location);

			req.url = location;
			doRoute();
		};

		function simpleResponse(code, body, content_type, extra_headers) {

			res.writeHead(code, (extra_headers || []).concat(
                           [ ["Content-Type", content_type],
                             ["Content-Length", Buffer.byteLength(body, 'utf8')]
                           ]));

			// don't write a body if the verb is head
			if (req.method !== "HEAD") { res.write(body, "utf8"); }

			res.end();
		};

		res.simpleText = function (code, body, extra_headers) {

			simpleResponse(code, body, "text/plain", extra_headers);
		};

		res.simpleHtml = function (code, body, extra_headers) {

			simpleResponse(code, body, "text/html", extra_headers);
		};

		res.simpleJson = function (code, json, extra_headers) {

			simpleResponse(code, JSON.stringify(json), "application/json", extra_headers);
		};

		res.onlyHead = function (code, extra_headers) {

			res.writeHead(code, (extra_headers || []).concat(
	                             [["Content-Type", "text/plain"]]));
			res.end();
		}

		res.notFound = function(req, res, message) {

			notFound(req, res, message);
		};

		function doRoute() {

			var uri = url_parse(req.url);
			var path = uri.pathname;

			logger("path: "+path);

			for (var i = 0, l = routes.length; i < l; i += 1) {

				var route = routes[i];

				if (req.method === route.method) {

					var match = path.match(route.pattern);
					
					if (match && match[0].length > 0) {

						logger("match: "+match);

						match.shift();

						match = match.map(function (part) {

							return part ? unescape(part) : part;
						});

						match.unshift(res);
						match.unshift(req);

						// body of request, put or post
						if (route.format !== undefined) {

							var body = "";
							req.setEncoding("utf8");

							req.addListener("data", function (chunk) {

								body += chunk;
							});

							req.addListener("end", function () {

								if (route.format === "json") {

									if(body) {

										try {

											body = JSON.parse(unescape(body));

										} catch(e) {

											logger("Error JSON: "+body);
											logger(e);

											body = {};
										}
									} else {

										body = {};
									}
								}

								// append to array of args
								match.push(body);

								route.handler.apply(null, match);
							});
	
							return;
						}

						var result = route.handler.apply(null, match);

						switch (typeof result) {

						case "string":
							res.simpleHtml(200, result);
							break;

						case "object":
							res.simpleJson(200, result);
							break;
						}

						return;
					}
				}
			}

			notFound(req, res);
		}

		doRoute();
	};

	return {
		get: get,
		post: post,
		put: put,
		del: del,
		head: head,
		resource: resource,
		serve: serve
	};

}

==> n-rest-1.1.0.tgz: package/n-rest.js <==
var url_parse = require("url").parse;

function notFound(req, res, message) {

	message = (message || "Not Found\n") + "";

	res.writeHead(404, {
		"Content-Type": "text/plain",
		"Content-Length": Buffer.byteLength(message, "utf8")
	});

	if (req.method !== "HEAD") res.write(message);

	res.end();
}

exports.Server = function(logger) {

	logger = logger || console.log;

	var routes = [];

	function addRoute(method, pattern, handler, format) {

		if (typeof pattern === 'string') {

			pattern = new RegExp("^" + pattern + "$");
		}

		var route = {
			method: method,
			pattern: pattern,
			handler: handler
		};

		if (format !== undefined) {

			route.format = format;
		}

		routes.push(route);
	};

	function get(pattern, handler) {

		return addRoute("GET", pattern, handler);
	};
	function post(pattern, handler, format) {

		return addRoute("POST", pattern, handler, format);
	};
	function put(pattern, handler, format) {

		return addRoute("PUT", pattern, handler, format);
	};
	function del(pattern, handler) {

		return addRoute("DELETE", pattern, handler);
	};
	function head(pattern, handler) {

		return addRoute("HEAD", pattern, handler);
	};

	function resource(name, controller, format) {

		var parts = name.split("/");
		var size = parts.length;

		var ic_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];

			if(i > 0 && size > 1) {

				ic_regexp += "/([^/]+)/";
			}

			ic_regexp += _part;		
		}
		ic_regexp += ";?([A-Za-z_]*)";

		var edit_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];

			if(i > 0 && size > 1) {

				edit_regexp += "/([^/]+)/";
			}

			edit_regexp += _part;
		};
		edit_regexp += "/([^/]+)/edit;?([A-Za-z_]*)";

		var newing_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];

			if(i > 0 && size > 1) {

				newing_regexp += "/([^/]+)/";
			}

			newing_regexp += _part;
		};
		newing_regexp += "/new;?([A-Za-z_]*)";

		var sud_regexp = "^/";
		for(var i=0;i<size;i++) {
			var _part = parts[i];

			if(i > 0 && size > 1) {

				sud_regexp += "/([^/]+)/";
			}

			sud_regexp += _part;
		};
		sud_regexp += "/([^/]);?([A-Za-z_]*)";

		logger("get: "+ic_regexp +" --> index");
		logger("get: "+sud_regexp +" --> show");
		logger("get: "+edit_regexp +" --> edit");
		logger("get: "+newing_regexp +" --> newing");
		logger("post: "+ic_regexp +" --> create");
		logger("put: "+sud_regexp +" --> update");
		logger("delete: "+sud_regexp +" --> destroy");

	    get(new RegExp(edit_regexp), controller.edit);
	    get(new RegExp(newing_regexp), controller.newing);
	    get(new RegExp(sud_regexp), controller.show);
	    get(new RegExp(ic_regexp), controller.index);
		post(new RegExp(ic_regexp), controller.create, format);
		put(new RegExp(sud_regexp), controller.update, format);
		del(new RegExp(sud_regexp), controller.destroy);
	};

	function serve(req, res) {

		res.redirect = function redirect(location) {

			res.writeHead(302, {"Location": location});
			res.end();
		};

		res.innerRedirect = function innerRedirect(location) {

			logger("Internal Redirect: " + req.url + " -> " + location);

			req.url = location;
			doRoute();
		};

		function simpleResponse(code, body, content_type, extra_headers) {

			res.writeHead(code, (extra_headers || []).concat(
                           [ ["Content-Type", content_type],
                             ["Content-Length", Buffer.byteLength(body, 'utf8')]
                           ]));

			// don't write a body if the verb is head
			if (req.method !== "HEAD") { res.write(body, "utf8"); }

			res.end();
		};

		res.simpleText = function (code, body, extra_headers) {

			simpleResponse(code, body, "text/plain", extra_headers);
		};

		res.simpleHtml = function (code, body, extra_headers) {

			simpleResponse(code, body, "text/html", extra_headers);
		};

		res.simpleJson = function (code, json, extra_headers) {

			simpleResponse(code, JSON.stringify(json), "application/json", extra_headers);
		};

		res.onlyHead = function (code, extra_headers) {

			res.writeHead(code, (extra_headers || []).concat(
	                             [["Content-Type", "text/plain"]]));
			res.end();
		}

		res.notFound = function(req, res, message) {

			notFound(req, res, message);
		};

		function doRoute() {

			var uri = url_parse(req.url);
			var path = uri.pathname;

			logger("path: "+path);

			for (var i = 0, l = routes.length; i < l; i += 1) {

				var route = routes[i];

				if (req.method === route.method) {

					var match = path.match(route.pattern);
					
					if (match && match[0].length > 0) {

						logger("match: "+match);

						match.shift();

						match = match.map(function (part) {

							return part ? unescape(part) : part;
						});

						match.unshift(res);
						match.unshift(req);

						// body of request, put or post
						if (route.format !== undefined) {

							var body = "";
							req.setEncoding("utf8");

							req.addListener("data", function (chunk) {

								body += chunk;
							});

							req.addListener("end", function () {

								if (route.format === "json") {

									if(body) {

										try {

											body = JSON.parse(unescape(body));

										} catch(e) {

											logger("Error JSON: "+body);
											logger(e);

											body = {};
										}
									} else {

										body = {};
									}
								}

								// append to array of args
								match.push(body);

								route.handler.apply(null, match);
							});
	
							return;
						}

						var result = route.handler.apply(null, match);

						switch (typeof result) {

						case "string":
							res.simpleHtml(200, result);
							break;

						case "object":
							res.simpleJson(200, result);
							break;
						}

						return;
					}
				}
			}

			notFound(req, res);
		}

		doRoute();
	};

	return {
		get: get,
		post: post,
		put: put,
		del: del,
		head: head,
		resource: resource,
		serve: serve
	};

}
